/**
   The MySensors Arduino library handles the wireless radio link and protocol
   between your home built sensors/actuators and HA controller of choice.
   The sensors forms a self healing radio network with optional repeaters. Each
   repeater and gateway builds a routing tables in EEPROM which keeps track of the
   network topology allowing messages to be routed to nodes.

   Created by Henrik Ekblad <henrik.ekblad@mysensors.org>
   Copyright (C) 2013-2015 Sensnology AB
   Full contributor list: https://github.com/mysensors/Arduino/graphs/contributors

   Documentation: http://www.mysensors.org
   Support Forum: http://forum.mysensors.org

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   version 2 as published by the Free Software Foundation.

 *******************************

   REVISION HISTORY
   Version 1.0 - Henrik EKblad

   DESCRIPTION
   This sketch provides an example how to implement a humidity/temperature
   sensor using SI702111/SI7021-22
   http://www.mysensors.org/build/humidity
*/

//#define DEBUG_PROGRAMM
#define SI7021ARRAY  2
//#define ZWEITER_EMPFAENGER  103  //5 Heizung   103 Geblaese

#include <MyTransportRFM69.h>
#include <SPI.h>
#include <MySensor.h>
#include <SI7021.h>

#define CHILD_ID_HUM 0
#define CHILD_ID_TEMP 1
#if defined (ZWEITER_EMPFAENGER)
#define CHILD_ID_REQUEST_TEMP2  6   //6=Eisenbahn 9=EG 10=OG
#define CHILD_ID_REQUEST_HUM2   7   //7=Eisenbahn 11=EG 12=OG
#endif
#define SENDEN_WIEDERHOLUNG 5
#define  VCC_100 300
#define VCC_MEASURE_TIME 8
#define MIND_VCC_SEND_TIME 90
#define SLEEP_TIME  900000L // Sleep time between reads (in milliseconds)
#define DIFFSEND_INNEN          15    //in Hundertstel
#define DIFFSEND_INNEN_HUM      300   //in Hundertstel
#define TRUE 0
#define FALSE 7
#define MANUELL 20
#define INNEN 0
#define INNEN_HUM 1
#define SEND_TEMP_HUM (1<<0)

//MyTransportRFM69 transport;
MyTransportRFM69 transport(RFM69_FREQUENCY, RFM69_NETWORKID, RF69_SPI_CS, RF69_IRQ_PIN, false, RF69_IRQ_NUM);
MyHwATMega328 hw;
MySensor gw(transport, hw);

SI7021 sensor;
int last_temperature[2];
int temperature[2];
byte temp_nicht_gesendet;
byte hum_nicht_gesendet;
boolean metric = true;
byte send_return;
byte SI7021_Read_true = FALSE;
byte send_to_zweiter_Empfaenger[2] = {false, false};


#ifdef SI7021ARRAY
byte arraypos;
int temparray[SI7021ARRAY];
int humarray[SI7021ARRAY];
#endif

int oldBatteryPcnt;
unsigned long lastVccMeasure = 0 - VCC_MEASURE_TIME;
unsigned long lastVccSend = 0 - MIND_VCC_SEND_TIME;
unsigned long cycles;
int diff_temperature_send[] = {DIFFSEND_INNEN, DIFFSEND_INNEN_HUM};
MyMessage msgHum(CHILD_ID_HUM, V_HUM);
MyMessage msgTemp(CHILD_ID_TEMP, V_TEMP);
#if defined (ZWEITER_EMPFAENGER)
MyMessage msgHum_2RX(CHILD_ID_REQUEST_HUM2, V_HUM);
MyMessage msgTemp_2RX(CHILD_ID_REQUEST_TEMP2, V_TEMP);
#endif

void setup()
{
#ifdef DEBUG_PROGRAMM
  Serial.begin(57600);
  Serial.print("Start");
#endif
  gw.begin();
  transport.setPowerLevel(27); //Power 27 + HW=false bestes Verhältnis Strom/Signal vgl.: http://blog.andrehessling.de/2015/02/07/figuring-out-the-power-level-settings-of-hoperfs-rfm69-hwhcw-modules/#more-11
#ifdef DEBUG_PROGRAMM
  Serial.print(".");
#endif
  // Send the Sketch Version Information to the Gateway
  gw.sendSketchInfo("Humidity Si7021", "1.0");
#ifdef DEBUG_PROGRAMM
  Serial.print(".");
#endif
  // Register all sensors to gw (they will be created as child devices)
  gw.present(CHILD_ID_HUM, S_HUM);
#ifdef DEBUG_PROGRAMM
  Serial.print(".");
#endif
  gw.present(CHILD_ID_TEMP, S_TEMP);
#ifdef DEBUG_PROGRAMM
  Serial.print(".");
#endif

  metric = gw.getConfig().isMetric;
#ifdef DEBUG_PROGRAMM
  Serial.println("fertig");
#endif
  analogReference(INTERNAL); // Set the aref to the internal 1.1V reference
  ACSR = (1 << ACD); //Disable Analog Comparator
  ADCSRA &= ~(1 << ADEN); // Disable Analog Digital Converter
  PRR |= (1 << PRADC);
  //  PRR |= (1 << PRTWI); //Disable TWI = I2C
  PRR |= (1 << PRTIM2);
#ifndef DEBUG_PROGRAMM
  PRR |= (1 << PRUSART0);
#endif
  //  DIDR0 = //Digital Input Disable
  //transport.powerDown();
  sensor.begin();
}

void loop()
{
  PRR &= ~(1 << PRTWI); //Enable TWI = I2C
  delay(2000);
  SI7021_lesen();
  PRR |= (1 << PRTWI); //Disable TWI = I2C
  while (send_return != 0) {
    for (int j = 0; j < 2; j++) {
      if ((send_return & (1 << j) ) == (1 << j) ) {
        senden(j);
        gw.wait(500);
        j = 0;
      }
    }
  }
  if (cycles - lastVccMeasure > VCC_MEASURE_TIME) {
    lastVccMeasure = cycles;
    unsigned int batteryPcnt = getVoltage();
#ifdef DEBUG_PROGRAMM
    Serial.print("Battery Voltage: ");
    Serial.print(batteryPcnt / 100);
    Serial.print(".");
    Serial.print(batteryPcnt % 100);
    Serial.print("V");
#endif
    batteryPcnt = batteryPcnt * 100 / VCC_100;
    if (oldBatteryPcnt > batteryPcnt + 1 || oldBatteryPcnt < batteryPcnt - 1 || cycles - lastVccSend > MIND_VCC_SEND_TIME) {
      // Power up radio after sleep
      lastVccSend = cycles;
      gw.sendBatteryLevel(batteryPcnt);
      oldBatteryPcnt = batteryPcnt;
    }
  }
  cycles++;
  hw.sleep(SLEEP_TIME); //sleep a bit
}


/**************************************************************************************/
/* Allows to get the real Vcc (return value * 100).                                   */
/**************************************************************************************/
int getVoltage()
{
  bitClear(PRR, PRADC);
  ADCSRA |= bit(ADEN); // Enable the ADC
  const long InternalReferenceVoltage = 1056L;
  ADMUX = (0 << REFS1) | (1 << REFS0) | (0 << ADLAR) | (1 << MUX3) | (1 << MUX2) | (1 << MUX1) | (0 << MUX0);
  delay(50);  // Let mux settle a little to get a more stable A/D conversion
  //Start a conversion
  ADCSRA |= _BV( ADSC );
  //Wait for it to complete
  while (((ADCSRA & (1 << ADSC)) != 0));
  //Scale the value
  int result = (((InternalReferenceVoltage * 1023L) / ADC) + 5L) / 10L;
  ADCSRA &= ~ bit(ADEN);
  bitSet(PRR, PRADC); // Disable the ADC to save power
  return result;
}


void SI7021_lesen() {
  int diff;
  int diff_hum;
  si7021_env data = sensor.getHumidityAndTemperature();
  int temp_temp =  data.celsiusHundredths;
  int temp_hum = data.humidityBasisPoints;
  if (temp_hum > 9999 || temp_hum < 0 || temp_temp > 5000 || temp_temp < -2000) {
    SI7021_Read_true++;
    if (temp_temp > 5000) temp_hum = 5000;
    if (temp_temp < -2000) temp_hum = -2000;
    if (temp_hum > 9999) temp_hum = 9999;
    if (temp_hum < 0) temp_hum = 0;
  } else {
    SI7021_Read_true = TRUE;
  }
  if (temp_temp > 9998) temp_temp = 9998;
#ifdef SI7021ARRAY
  arraypos++;
  if (arraypos == SI7021ARRAY) arraypos = 0;
  temparray[arraypos] = temp_temp;
  humarray[arraypos] = temp_hum;
  if (temparray[0] == 0) { //am Anfang Array komplett belegen
    for (int i = 0; i < SI7021ARRAY; i++) {
      temparray[i] = temparray[1];
      humarray[i] = humarray[1];
    }
  }
  temp_temp = temparray[0];
  temp_hum = humarray[0];
  for (int i = 1; i < SI7021ARRAY; i++) {
    temp_temp += temparray[i];
    temp_hum += humarray[i];
  }
  temp_temp = temp_temp / SI7021ARRAY;
  temp_hum = temp_hum / SI7021ARRAY;
#endif
  temperature[INNEN] = temp_temp;
  temperature[INNEN_HUM] = temp_hum;
  diff = temperature[INNEN] - last_temperature[INNEN];
  diff_hum = temperature[INNEN_HUM] - last_temperature[INNEN_HUM];
  if (abs(diff) > diff_temperature_send[INNEN] || abs(diff_hum) > diff_temperature_send[INNEN_HUM] || temp_nicht_gesendet > SENDEN_WIEDERHOLUNG) {
    temp_nicht_gesendet = 0;
    send_return |= SEND_TEMP_HUM;
#if defined(DEBUG_PROGRAMM)
    Serial.print(F("Temp:"));
    Serial.print(temperature[INNEN]);
    Serial.print(F("  Hum:"));
    Serial.println(temperature[INNEN_HUM]);
#endif
  } else {
    temp_nicht_gesendet++;
#if defined(DEBUG_PROGRAMM)
    Serial.println (F("Temp/Hum gleich"));
#endif
  }
  if (SI7021_Read_true == TRUE) {
    //    requestMillis[AUSSEN_TEMP_REQUEST] = currentMillis;
  } else if (SI7021_Read_true >= FALSE) {
    //    request_attempt[AUSSEN_TEMP_REQUEST] = 1;
    if (SI7021_Read_true > FALSE + 2) SI7021_Read_true = FALSE + 2;
  }
}


void senden(int i) {
#if defined (DEBUG_PROGRAMM)
  Serial.print(cycles);
  Serial.print(F(" senden:"));
  Serial.print(i);
  Serial.print(F(" send_return:"));
  Serial.println(send_return, BIN);
#endif
  send_return &= ~(1 << i);
  float temp_send;
  switch (i) {
    case INNEN:
      if (send_to_zweiter_Empfaenger[i] == false) {
        temp_send = int(temperature[i]/10) * 0.1;
        last_temperature[i] = temperature[i];
        gw.send(msgTemp.set(temp_send, 1));
#if defined (DEBUG_PROGRAMM)
        Serial.println(F("TempSend"));
#endif
        gw.wait(500);
        temp_send = int(temperature[i + 1]/10) * 0.1;
        last_temperature[i + 1] = temperature[i + 1];
        gw.send(msgHum.set(temp_send, 1));
#if defined (DEBUG_PROGRAMM)
        Serial.println(F("HumSend"));
#endif
#ifdef ZWEITER_EMPFAENGER
        send_to_zweiter_Empfaenger[i] = true;
        send_return |= (1 << i);
#endif
      } else {
#if defined (DEBUG_PROGRAMM)
        Serial.print(F("2.Empfaenger "));
#endif
#ifdef ZWEITER_EMPFAENGER
        temp_send = int(temperature[i]/10) * 0.1;
        gw.send(msgTemp_2RX.setDestination(ZWEITER_EMPFAENGER).set(temp_send, 1));
        send_to_zweiter_Empfaenger[i] = false;
#if defined (DEBUG_PROGRAMM)
        Serial.println(F("TempSend 2.Empf"));
#endif
        gw.wait(500);
#endif
#ifdef ZWEITER_EMPFAENGER
        temp_send = int(temperature[i + 1]/10) * 0.1;
        gw.send(msgHum_2RX.setDestination(ZWEITER_EMPFAENGER).set(temp_send, 1));
        send_to_zweiter_Empfaenger[i] = false;
#if defined (DEBUG_PROGRAMM)
        Serial.println(F("HumSend 2.Empf"));
#endif
#endif
      }
      break;
  }
}



